########################################################################
####################                EXERCISE              ##############
####################        Differential Expression       ##############
####################         SYNTHETIC  Dataset           ##############
########################################################################

# By Sonia Tarazona
# Last modified: 16-III-2011



source("BetaNOISeq.r")

library(DEGseq)
library(edgeR)
library(DESeq)
library(baySeq)




## LOADING DATA (simulated dataset from baySeq package)

# deg = first 100; condition1 = columns 1-5; condition2 = columns 6-10
simCount <- read.delim("simCount.txt", row.names = 1, header = TRUE)
head(simCount)

depth <- colSums(simCount)  # computing sequencing depth for each replicate




###########################################################################
###########################################################################


###########       NOISeq      #############################################



### NOISeq-real (using available replicates to estimate noise distribution)

res1noiseq <- noiseq(simCount[,1:5], simCount[,6:10], nss = 0, q = 0.8) 
# nss is the number of simulated replicates to be computed, 0 in this case
# (because available replicates are to be used)
# q = 0.8, threshold for the probability of a gene to be differentially expressed

names(res1noiseq)

head(res1noiseq$probab)  # probability of differential expression for each gene
res1noiseq$deg           # differentially expressed genes

# Ms and Ds are M and D values for signal
# Mn and Dn are M and D values for noise

# To know if genes are over-expressed in condition 1 or in condition2
# we can look at M-values for deg
# If M > 0 -> over-expressed in condition 1
# If M < 0 -> over-expressed in condition 2
res1noiseq$Ms[res1noiseq$deg,]



#------------------------------------------------------------------------------#



### NOISeq-sim (simulating replicates)

res2noiseq <- noiseq(simCount[,1:5], simCount[,6:10], q = 0.8, pnr = 0.2, nss = 5)
# We have estimated noise distribution by simulating 5 replicates of each condition.
# The total number of reads in each simulated replicate is equal to the total number
# of reads in the corresponding condition.

names(res2noiseq)

res2noiseq$deg           # differentially expressed genes

# To know if genes are over-expressed in condition 1 or in condition2
# we can look at M-values for deg
# If M > 0 -> over-expressed in condition 1
# If M < 0 -> over-expressed in condition 2
res2noiseq$Ms[res2noiseq$deg,]



#----------------------------------------------------------------------#




###########       DEGseq      #############################################


# Preparing data for DEGseq package
geneExpM1 <- data.frame(rownames(simCount),simCount[,1:5])
geneExpM2 <- data.frame(rownames(simCount),simCount[,6:10])


### MARS (MA-plot based method)

DEGexp(geneExpMatrix1 = geneExpM1, geneCol1 = 1, expCol1 = 2:6, depth1 = depth[1:5],
       groupLabel1 = "group1", geneExpMatrix2 = geneExpM2, geneCol2 = 1, expCol2 = 2:6,
       depth2 = depth[6:10], groupLabel2="group2", method = "MARS", qValue = 1e-3,
       thresholdKind = 3, outputDir = "/home/biouser/Desktop/", normalMethod = "none",
       rawCount = TRUE)

res1mars <- read.delim(file = "/home/biouser/Desktop/output_score.txt")

res1mars.deg <- as.character(res1mars[which(res1mars[,8] < 0.05),1]) # deg


#----------------------------------------------------------------------#



### LRT (Likelihood Ratio Test from Marioni et al., 2008)

DEGexp(geneExpMatrix1 = geneExpM1, geneCol1 = 1, expCol1 = 2:6, depth1 = depth[1:5],
       groupLabel1 = "group1", geneExpMatrix2 = geneExpM2, geneCol2 = 1, expCol2 = 2:6,
       depth2 = depth[6:10], groupLabel2="group2", method = "LRT", qValue = 1e-3,
       thresholdKind = 3, outputDir = "/home/biouser/Desktop/", normalMethod = "none",
       rawCount = TRUE)

res1lrt <- read.delim(file = "/home/biouser/Desktop/output_score.txt")

res1lrt.deg <- as.character(res1lrt[which(res1lrt[,7] < 0.05),1])  # deg


#----------------------------------------------------------------------#



### FET (Fisher's Exact Test)

DEGexp(geneExpMatrix1 = geneExpM1, geneCol1 = 1, expCol1 = 2:6, depth1 = depth[1:5],
       groupLabel1 = "group1", geneExpMatrix2 = geneExpM2, geneCol2 = 1, expCol2 = 2:6,
       depth2 = depth[6:10], groupLabel2="group2", method = "FET", qValue = 1e-3,
       thresholdKind = 3, outputDir = "/home/biouser/Desktop/", normalMethod = "none",
       rawCount = TRUE)

res1fet <- read.delim(file = "/home/biouser/Desktop/output_score.txt")
res1fet.deg <- as.character(res1fet[which(res1fet[,7] < 0.05),1])  # deg



#----------------------------------------------------------------------#




###########       edgeR        #############################################

# Preparing data for edgeR package
datsim <- DGEList(counts = simCount, group = rep(1:2, each = 5))

# Common Dispersion
d <- estimateCommonDisp(datsim)
d <- exactTest(d)

res1edgeR <- topTags(d, n = 1000)[[1]]

res1edgeR.deg <- rownames(res1edgeR)[which(res1edgeR$FDR < 0.05)] # deg


# Tagwise Dispersion
d2 <- estimateTagwiseDisp(estimateCommonDisp(datsim), prior.n = 10)
d2 <- exactTest(d2, common.disp = FALSE)
res1edgeR2 <- topTags(d2, n = 1000)[[1]]

res1edgeR2.deg <- rownames(res1edgeR2)[which(res1edgeR2$FDR < 0.05)] # deg



#----------------------------------------------------------------------#




###########       DESeq        #############################################

# Preparing data for DESeq package
gr <- factor(rep(1:2, each = 5), labels = c("cond1", "cond2"))
cds <- newCountDataSet(simCount, gr)  # counts

cds1 <- estimateSizeFactors(cds)  # library relative sizes
cds2 <- estimateVarianceFunctions(cds1) # variance estimation

res1deseq <- nbinomTest(cds2, "cond1", "cond2")

res1deseq.deg <- res1deseq$id[which(res1deseq$padj < 0.05)] # deg


#----------------------------------------------------------------------#



###########       baySeq        #############################################

# Preparing data for baySeq package
cl <- NULL  # when we don't want to use clusters (parallelization)
grs <- list(NDE = c(1, 1, 1, 1, 1, 1, 1, 1, 1, 1), DE = c(1, 1, 1, 1, 1, 2, 2, 2, 2, 2))
CD <- new("countData", data = as.matrix(simCount), replicates = rep(1:2, each = 5),
          libsizes = depth, groups = grs)
CD@annotation <- data.frame(name = paste("count", 1:1000, sep = "_"))

## Poisson
CDP.Poi <- getPriors.Pois(CD, samplesize = 20, takemean = TRUE, cl = cl)  # Poisson
CDPost.Poi <- getLikelihoods.Pois(CDP.Poi, pET = "BIC", cl = cl)
CDPost.Poi@estProps  # estimated proportion of deg

res1bayseq <- exp(CDPost.Poi@posteriors) # because likelihoods are given in log-scale
rownames(res1bayseq) <- rownames(simCount)

res1bayseq.deg <- names(which(res1bayseq[,2] > 0.8)) # deg for threshold of 0.8


## Negative binomial
CDP.NBML <- getPriors.NB(CD, samplesize = 1000, estimation = "QL", cl = cl) # NB
CDPost.NBML <- getLikelihoods.NB(CDP.NBML, pET = "BIC", cl = cl)
CDPost.NBML@estProps  # estimated proportion of deg

res2bayseq <- exp(CDPost.NBML@posteriors)
rownames(res2bayseq) <- rownames(simCount)

res2bayseq.deg <- names(which(res2bayseq[,2] > 0.8)) # deg



#----------------------------------------------------------------------#

